require "adjusted_view"

local function setupCall()
  if ActiveCall ~= nil then
    return true
  end
    
  ActiveCall = tango.videophone.Call_getActiveCall()
  if ActiveCall == nil then
    print("setupVideoView() we are not in a call, return")
    return false
  end  

  return true
end  

local function getVideoControl() 
  setupCall()
  return ActiveCall and ActiveCall:getVideoStreamsControl() or nil
end

local function getAudioControl()
  setupCall()
  return ActiveCall and ActiveCall:getAudioStreamsControl() or nil
end  


------------------------
-- exit icon
------------------------
function setupExitIcon(adjustedView)
  ExitIcon = Button {
    name = "ExitIcon",
    text = "",
    parent = adjustedView:getView(),
    styles = {CLOSE_BUTTON_STYLES},
    skinResizable = false,      
    size = { ICON_WIDTH, ICON_WIDTH },
    pos = { adjustedView:getAdjustedPos(0 , 0) }
  }
  return ExitIcon
end

------------------------
-- Add More icon
------------------------
function setupAddMoreIcon(adjustedView)
  AddMoreIcon = Button {
    name = "AddMoreIcon",
    text = "",
    parent = adjustedView:getView(),
    styles = {ADD_MORE_ICON_NORMAL_STYLES},
    skinResizable = false,      
    size = { ICON_WIDTH, ICON_WIDTH },
    pos = { adjustedView:getAdjustedPos(adjustedView:getAdjustedViewWidth() - ICON_WIDTH, 0) }
  }
  return AddMoreIcon
end  
  
------------------------
-- audio  Manipulation
------------------------
function setupMuteButton(adjustedView)
  print("call_control setupMuteButton() Enter")
  if not setupCall() then
    return
  end  
  local view = adjustedView:getView()
	MuteButton = Button {
    name = "MuteButton",
    text = "",
    parent = view,
    styles = {MUTE_BUTTON_STYLES},
    skinResizable = false,
  }

  MuteButton:setToggle(true)
  MuteButton:setOnButtonDown(onMuteClick)
  MuteButton:setOnButtonUp(onMuteClick)
  
  local x = adjustedView:getAdjustedViewWidth() - PIP_MARGIN - MuteButton:getWidth()
  local y = adjustedView:getAdjustedViewHeight() - PIP_MARGIN - MuteButton:getHeight()
  x, y = adjustedView:getAdjustedPos( x, y )
  MuteButton:setPos(x, y )
  MuteButton:setSelected(getAudioControl():isMuted())
  print("call_control setupMuteButton() Exit")
  return MuteButton
end  

function onMuteClick(event)
  print("call_control onMuteClick()")
  local nextState = not getAudioControl():isMuted()
  print("call_control change Mute status to muted: " .. toString(nextState))
  getAudioControl():setMute(nextState)
end

------------------------
-- Video  Manipulation
------------------------
WasSending = false
WasReceiving = false
PIP_CORNER = TangoPip.BOT_LEFT
 
function getVideoDirection()
  return WasSending, WasReceiving
end  

function setupVideoView(adjustedView, pipLayer)
  print("call_control setupVideoView() Enter")
  local view = adjustedView:getView()
  if DEBUG_DISABLE_VIDEO then
    return 
  end
  
  if not setupCall() or getVideoControl() == nil then
    return
  end 
  
  WasSending, WasReceiving = getVideoControl():isSendingInPrimaryCallUI(), getVideoControl():isReceivingInPrimaryCallUI() 
  
  if (not WasSending and not WasReceiving) then
    -- Fix DRGN-3008  [Photo Sharing] iPad 2 doesn't PIP if camera and photo share opened same time
    print("call_control setupVideoView() we were not in a video call when starting photo sharing." )
  end  
  
  -- send video
  resumeSendingVideo()
  
  -- create pip
  local videoTexture = nil
  PipTextureType = MOAIVideoTexture.INVALID
  if MOAIVideoTexture then
    print("call_control Create video in texture")
    videoTexture = MOAIVideoTexture.new ()
    PipTextureType = (WasReceiving or not WasSending) and MOAIVideoTexture.PLAYBACK or  MOAIVideoTexture.PREVIEW
    videoTexture:setType (PipTextureType)
  end
  	
	local pipWidth = Application.screenWidth * 0.3
  peerPip = TangoPip {
    layer = pipLayer,
    texture = videoTexture,
    pos = {0, 0},
    size = { pipWidth, pipWidth *1.5},

    dragBegan = function()
      view:getLayer():setTouchEnabled(false)
    end,
    
    -- swap mute icon if pip is overlapping
    dragEnded = function(corner)
      local adjustedCorner = adjustedView:getAdjustedCorner(corner)
      PIP_CORNER = adjustedCorner
      
      print("pip moved to " .. adjustedCorner)
      local viewWidth, viewHeight = adjustedView:getAdjustedViewSize()
      
      -- handle the collision with mute button
      if(adjustedCorner == TangoPip.BOT_RIGHT) then
        MuteButton:setPos(adjustedView:getAdjustedPos(PIP_MARGIN, viewHeight - PIP_MARGIN - MuteButton:getHeight()))        
      else
        MuteButton:setPos(adjustedView:getAdjustedPos(viewWidth - PIP_MARGIN - MuteButton:getWidth(), viewHeight - PIP_MARGIN - MuteButton:getHeight()))        
      end
      
      view:getLayer():setTouchEnabled(true)      
    end,
    
    shouldMoveToNewCorner = function(corner)
      local adjustedCorner = adjustedView:getAdjustedCorner(corner)
      return adjustedCorner ~= TangoPip.TOP_LEFT and  adjustedCorner ~= TangoPip.TOP_RIGHT
    end  
  }
  
  peerPip:setVisible(false)
  pipLayer:setTouchEnabled(false)
  
  local corner = adjustedView:getPhysicalCorner(PIP_CORNER)
  print("physical corner for " .. PIP_CORNER .. " is " .. corner)
  peerPip:moveToCorner(corner)
  peerPip.dragEndedCB(corner) -- handle the collision with the mute button
  
  local function onSendChanged(sending)
    print("call_control onSendChanged(): " .. toString(sending))
    if PipTextureType == MOAIVideoTexture.PREVIEW then
      peerPip:setVisible(sending)    
      if sending then
        pipLayer:setTouchEnabled(true)
      end  
    end  
  end

  local function onReceiveChanged(receiving)
    print("call_control onReceiveChanged(): " .. toString(receiving))
    if PipTextureType == MOAIVideoTexture.PLAYBACK then
      peerPip:setVisible(receiving)
      if receiving then
        pipLayer:setTouchEnabled(true)
      end
    end   
  end
  
  startVideoEvents(onSendChanged, onReceiveChanged)
  
  print("call_control setupVideoView() Exit")
  return peerPip
end  

function resumeSendingVideo()
  if getVideoControl() == nil then
    return
  end 
  local supportSendingVideo = getVideoControl():isSendingSupported()
  if supportSendingVideo then
    if WasSending and not getVideoControl():isSending() then
      print("call_control resumeSendingVideo()")
      getVideoControl():startSending()  
    end
  else
    print ('OUTGOING VIDEO STREAM IS *NOT* SUPPORTED')
  end
end

function stopSendingVideo()
  if getVideoControl() and getVideoControl():isSending() then
    print("call_control stopSendingVideo()")
    getVideoControl():stopSending()  
  end
end
------------------------
-- Video  Events
------------------------

function startVideoEvents(onSend, onReceive)
  
  if(RunVideoEventCr or DEBUG_DISABLE_VIDEO) then
    print "startVideoEvents cancelled"
  end
  
  VideoEventOnSend = onSend
  VideoEventOnReceive = onReceive
  
  videoEventCr = MOAICoroutine.new()
  videoEventCr:run ( reportVideoOn )
  
end

function stopVideoEvents()
  RunVideoEventCr = false
end

function reportVideoOn ()
  
  local lastOut = false
  local lastIn = false
  RunVideoEventCr = true
  
  while RunVideoEventCr do
    if getVideoControl() then
      local out = getVideoControl():isSending()
      local inc = getVideoControl():isReceiving()

      if(out ~= lastOut) then
        VideoEventOnSend(out)
        lastOut = out
      end
      if(inc ~= lastIn) then
        VideoEventOnReceive(inc)
        lastIn = inc
      end
    end
    
    coroutine:yield ()
  end
end


