local table             = require "hp/lang/table"
local class             = require "hp/lang/class"
local Event             = require "hp/event/Event"
local Component         = require "hp/gui/Component"

require "global_variables"
require "utilities"

-- class define
local M                 = class(Component)
local super             = Component
function M:initInternal()
  super.initInternal(self)
end

function M:createChildren()
  printf("intro view Create children.")
  self._isInitiator = false
  self._isSlowDevice = true
  self._selectedPhotoCount = 0
  -- initialize background
  local backgroundTexture = MOAITexture.new ()
  backgroundTexture:load ( "../assets/bg_texture.png" )
  backgroundTexture:setWrap ( true )
  backgroundTexture:setFilter(MOAITexture.GL_LINEAR,MOAITexture.GL_LINEAR)
  self._bgSprite = Sprite {
    parent = self,
  }
  self._bgSprite:setTexture(backgroundTexture)
  
  -- title
  self._title = TextLabel {
    parent = self,
    textSize = 45,
  }
  self._title:setRGBA(101, 100, 99, 1)
  self._title:setAlignment(TextLabel.HORIZOTAL_ALIGNS.center, TextLabel.VERTICAL_ALIGNS.center)
  
  -- Version Label
  self._Versionlabel = TextLabel {
    text = DEBUG_VERSION_NUMBER,
    textSize = 30,
  }
  self._Versionlabel:setRGBA(95, 94, 92, 1)
  self._Versionlabel:setAlignment(TextLabel.HORIZOTAL_ALIGNS.center, TextLabel.VERTICAL_ALIGNS.center)
  if DEBUG_SHOW_VERSION_NUMBER then
    print("show version number " .. DEBUG_VERSION_NUMBER)
    self._Versionlabel:setParent(self)
  end  
end

function M:setIsSlowDevice(slow)
  self._isSlowDevice = slow
end  

function M:setIsInitiator(isInitiator)
  print("intro view setIsInitiator " .. toString(isInitiator))
  self._isInitiator = isInitiator  
  if isInitiator then
    self._title:setText(localize("sp_share_photos"))
    
    self._loadContent = Sprite {
      parent = self,
      texture = "../assets/photo_icon_sharing.png"
    }

    self._label = TextLabel {
      text = "",
      parent = self,
      textSize = 30,
    }
    self._label:setRGBA(125, 124, 122, 1)
    self._label:setAlignment(TextLabel.HORIZOTAL_ALIGNS.center, TextLabel.VERTICAL_ALIGNS.center)
    self._label:setText(localize("sp_intro_view_sender_tip"))
    -- button  
    self.shareButton = Button {
      name = "Share Your Photos",
      text = localize("sp_intro_view_button"),
      parent = self,
      size = { 0, BUTTON_HEIGHT },
      styles = { SHARE_MORE_BUTTON_STYLES }
    }
  else
    self._countLabel = TextLabel {
      text = "",
      parent = self,
      textSize = 160,
    }
    self._countLabel:setRGBA(125, 124, 122, 1)
    self._countLabel:setAlignment(TextLabel.HORIZOTAL_ALIGNS.center, TextLabel.VERTICAL_ALIGNS.center)
    self._loadContentSprites = {}
    for index = 1, 3 do 
      self._loadContentSprites[index] = Sprite {
        parent = self,
        texture = "../assets/loading_content_icon_lg@2x.png"
      }
      self._loadContentSprites[index]:setColor(0, 0, 0, 0)
    end
  end  
end

function M:setBottomLine(bottomLine)
  self._contentBottomLine = bottomLine
end  

function M:setPeerName(name)
  self._peerName = name
  if not self._isInitiator then
    self._title:setText(self._peerName .. "\n" .. localize("sp_intro_view_receiver_tip"))
  end  
end

function M:resizeHandler(e)
  if self._contentBottomLine == nil then
    return
  end
  print("intro view resizeHandler()")

  self._bgSprite.deck:setUVRect(0, 0, self:getWidth() /100 , -self:getHeight()/100)
  self._bgSprite:setSize(self:getWidth(), self:getHeight())
  
  self._Versionlabel:setSize(self:getWidth(), 40)
  self._Versionlabel:setPos(0, self:getHeight() - self._Versionlabel:getHeight() - 50)
  
  if self._isInitiator then
    -- not related to bottomLine
    self._title:setSize(self:getWidth(), 100)
    self._loadContent:setLeft((self:getWidth() - self._loadContent:getWidth()) / 2)
    self._label:setSize(400, 100)
    self._label:setLeft((self:getWidth() - self._label:getWidth())/2)
    self.shareButton:setWidth(self:getWidth() < 1000 and (self:getWidth() - 100) or 600)
    self.shareButton:setLeft((self:getWidth() - self.shareButton:getWidth()) / 2);

  
    -- recalculate self._contentBottomLine
    local ratio = 0.65 -- expeted every content start here
    if self:getHeight()  * ratio < self._contentBottomLine  then
      self._contentBottomLine = self:getHeight()  * ratio
    end
    
    -- layout the components from bottom up
    self.shareButton:setBottom(self._contentBottomLine)
    self._label:setBottom(self.shareButton:getTop() - 33) 
    self._loadContent:setBottom(self._label:getTop() - 33)
    self._title:setBottom(self._loadContent:getTop() - 45)
  else
    self._title:setTextSize(40)
    self._title:setSize(self:getWidth(), 150)
    self._countLabel:setSize(self:getWidth(), 200)
    self._countLabel:setLeft((self:getWidth() - self._countLabel:getWidth())/2)
    
    -- recalculate self._contentBottomLine
    local ratio = 0.55 -- expeted every content start here
    if self:getHeight()  * ratio < self._contentBottomLine  then
      self._contentBottomLine = self:getHeight()  * ratio
    end
    
     -- layout the components from bottom up
    self:resetSpritesPosition()
    self._countLabel:setBottom(self._loadContentSprites[1]:getTop() - 50)
    self._title:setBottom(self._countLabel:getTop() - 20)
    
    if self._selectedPhotoCount == 0 then
      self:startLoopAnimation()
    end  
  end
end

function M:resetSpritesPosition()
  for index = 1, 3 do 
   self._loadContentSprites[index]:setBottom(self._contentBottomLine)
  end
  local spritesWidthInTotal = 400
  local gapBetweenSprites = (spritesWidthInTotal - 3*self._loadContentSprites[1]:getWidth()) / 2
  self._loadContentSprites[1]:setLeft((self:getWidth() - spritesWidthInTotal)/2)
  self._loadContentSprites[2]:setLeft(self._loadContentSprites[1]:getLeft() + self._loadContentSprites[1]:getWidth() + gapBetweenSprites)
  self._loadContentSprites[3]:setLeft(self._loadContentSprites[2]:getLeft() + self._loadContentSprites[2]:getWidth() + gapBetweenSprites)
end

function M:setPhotoCount(count)
  print("introView setPhotoCount() " .. count)
  self._selectedPhotoCount = count
  if not self._isInitiator and self._countLabel then
    self._countLabel:setText(count == 0 and "" or ("" .. count))
    if count == 0 then
      return
    else  
      self:stopAnimation()
      self:resetSpritesPosition()
    end
     
    local index = ((count % 3) == 0 and 3) or (count % 3)
    for i = 1, 3 do
      if i <= index then
        self._loadContentSprites[i]:setColor(1, 1, 1, 1)
      else        
        self._loadContentSprites[i]:setColor(0, 0, 0, 0)
      end
    end
    
    if not self._isSlowDevice then
      local slideRight = self:getWidth() - self._loadContentSprites[index]:getLeft()
      self._loadContentSprites[index]:setLeft(self:getWidth())
      self._animation = Animation({self._loadContentSprites[index]}, 0.5):moveLoc(-slideRight, 0, 0)
      self._animation:play()
    end
  end
end

function M:startLoopAnimation()
  print("introView startLoopAnimation() ")
  if self._animation == nil then
    self._animation = Animation():loop(0, 
        Animation():sequence( 
                  Animation(self._loadContentSprites[1]):wait(1.5):setColor(1, 1, 1, 1),
                  Animation(self._loadContentSprites[2]):wait(1.5):setColor(1, 1, 1, 1),
                  Animation(self._loadContentSprites[3]):wait(1.5):setColor(1, 1, 1, 1),
                  Animation({self._loadContentSprites[1], 
                             self._loadContentSprites[2],
                             self._loadContentSprites[3], 
                            }):wait(1):setColor(0, 0, 0, 0)
                  )
     )
  end             
  self._animation:play()
end

function M:stopAnimation()
  if(self._animation ~= nil) then
    print("introView stopAnimation() ")
    self._animation:stop()
    self._animation = nil
  end
end

function M:clearTexture()
  print("intro view clearTexture()")
  self._bgSprite:clearTexture()
end  


return M