-- main_scene.lua
module(..., package.seeall)

require "global_variables"
require "utilities"
AdjustedView = require "adjusted_view"
require "call_controls"
UIIntroView = require "intro_view"
UILoadingView = require "loading_view"
UIProgressBar = require "modules/progress_bar"
UISimpleDialog = require "modules/simple_dialog"
UIRoundedTextLabel = require "modules/rounded_corner_text_label"

------------------------
-- COMPLETED TASKS -----
------------------------


------ TODO

-- Tasks
-- font
-- 
------------------------

------------------------
-- Setup && Cleanup
------------------------
function setup()
    print("setup() with asset " .. DEBUG_VERSION_NUMBER)
    
    EnterTime = os.time()
    
    -- register the exit callback
    MOAISim.setPreDeleteContextCallback(beforeExit)
    
    --MOAISim.clearLoopFlags(MOAISim.SIM_LOOP_ALLOW_SPIN);

    if MOAIApp and not DEBUG_TRUN_OFF_BACK_BUTTON_INTERCEPTION then
      MOAIApp.setListener ( MOAIApp.BACK_BUTTON_PRESSED, onUserPressBackButton )
    end
    
    -- specify file paths
    local RootDir =  MOAIEnvironment.cacheDirectory
    assert(not isEmpty(RootDir))
    ImageDirName =  "image_dir" 
    AbsoluteImageDirPath = RootDir .. "/" .. ImageDirName
    print("AbsoluteImageDirPath = " .. AbsoluteImageDirPath)
    
    -- clean up just in case that previous session was terminated by accident
    cleanup()
    
    -- setup for this session
    setupOSType()
    setupDeviceType()
    setupTargetScreenSize()
    setupPeerInfo()
    setupOutputDir() 
    setupTransferListener()
    setupImagePicker()
    observeOrientation(scene)
end

function beforeExit()
  sendLog()
  cleanup()
end  

function sendLog()
  print("sendLog()")
  local duration = os.time() - EnterTime
  local wasSendingVideo, wasReceivingVideo = getVideoDirection()
  local inVideoCall = wasSendingVideo or wasReceivingVideo
  local bidirection = PhotoCountFromMe > 0 and PhotoCountToMe > 0
  printTable( tango.logger)
  local values = tango.logger.KeyValueCollection_createRawPointer() 
  values:add("share_photo_event_type", "end_session")
  values:add("photo_count", toString(CurrentPhotoCount))
  values:add("photo_transfered", toString(getPhotoSuccessReceivedCount()))
  values:add("photo_from_me", toString(PhotoCountFromMe))
  values:add("photo_to_me", toString(PhotoCountToMe))
  values:add("bidirection", toString(bidirection))
  values:add("viewed_all_photo_pages", toString(LogViewedAllPhotos))
  values:add("exit_page", toString(CurrentPage))
  values:add("duration", toString(duration))
  values:add("add_more", toString(LogBeenShareMorePhoto))
  values:add("in_video_call", toString(inVideoCall))
  values:add("lang", LANGUAGE)
  values:add("ever_click_pick_photo_button", toString(LogClickPickPhotoButton))
  values:add("is_initiator", toString(isInitiator()))
  values:add("exit_by", toString(LogExitBy))
  values:add("asset_version", DEBUG_VERSION_NUMBER)
  values:add("exited_while_picking_photos", toString(ImagePickerVisible))

  local logger = tango.logger.FeedbackLogger_getLogger()
  logger:logUIEventWithRawPointer(values)
end

function cleanup()
    print("cleanup() ENTER")
    MOAIImagePicker.dismissPicker()
    MOAIImagePicker.ignorePendingImages()
    cleanUpOutputDir()
    cleanupTransferListener()
    print("cleanup() EXIT")
end 

function setupDeviceType()
  local model = MOAIEnvironment.devFullModel
  print ("Device Model : " .. toString(model))
  for i,v in pairs(LOW_SPEED_DEVICES) do
    if string.starts(model, v) then
      print("This is a low speed device.")
      IsLowSpeedDevice = true
      break
    end  
  end
end  

------------------------
-- User actions
------------------------
function onUserPickingPhoto(count)
   print("onUserPickingPhoto() count: " .. count)
   p2pSendMessage(PICK_PHOTO_MESSAGE, count)
end  

function onUserAddPhoto(id, fullPath, imageData)
  LogViewedAllPhotos = false
  PhotoCountFromMe = PhotoCountFromMe + 1
  fullPath = fullPath or ""
  assert(not isEmpty(id))
  print("onUserAddPhoto() id:" .. id .. " path:" .. fullPath)
  
  CurrentPhotoCount = CurrentPhotoCount + 1
  
  local photo = {}
  photo[ID] = id
  photo[URL] = ""
  photo[STATUS] = STATUS_IDLE
  photo[DIRECTION] = DIRECTION_SENDER
  photo[PATH] = fullPath
  photo[PAGE] = CurrentPhotoCount
  photo[IMAGE_SPRITE] = nil
  photo[LOADING_VIEW] = nil
  photo[IMAGE_THUMBNAIL] = nil
  photo[PHOTO_LOADED] = false
  PhotoTable[photo[PAGE]] = photo
  
  if CurrentPage == 0 then
    CurrentPage = 1
    uiUpdatePhotoStatus()
  end
  
  uiCreatePhotoContainer(photo)
  
  uploadNextPhoto()
    
  return photo
end

function onUserPhotoGenerated(photo, fullPath, imageData)
  assert(photo and type(photo) == "table")
  assert(not isEmpty(fullPath))
  print("onUserPhotoGenerated() id:" .. photo[ID] .. " path:" .. fullPath)
  
  photo[PATH] = fullPath
  
  uiUpdatePhoto(photo, imageData)
  
  uploadNextPhoto()
  
end

function onUserScrollPhotoBegin(page)
  print("onUserScrollPhotoBegin() page:" .. page)  
  if page == CurrentPhotoCount then
    LogViewedAllPhotos = true
  elseif page >= CurrentPhotoCount + 1 then
    return
  end
  
  PointerSprite:stopAnimation()

  CurrentPage = page
  CurrentScrollId = CurrentScrollId + 1

  local photo = findPhotoByPage(page)  
    
  if photo then
    p2pSendMessage(SCROLL_PHOTO_MESSAGE, photo)
  else  
    print("ERORR: can not find photo with page " .. page)       
  end
  
  uiUpdatePhotoStatus()    
end

function onUserScrollPhotoEnded(page)
  print("onUserScrollPhotoEnded() page:" .. page)
  uiUpdateAdjacentPhotos()
end

function onUserScrollPhoto(page)
  print("onUserScrollPhoto() page:" .. page)  
  onUserScrollPhotoBegin(page)
  onUserScrollPhotoEnded(page)
end

function onPeerReceiveMessage(t)
  assert(type(t) == "table")
  local messageType = t[MESSAGE_TYPE] or ""
  print("onPeerReceiveMessage() " .. messageType)
  if messageType == STOP_SEND_VIDEO_MESSAGE then
    stopSendingVideo()
  elseif messageType == RESUME_SEND_VIDEO_MESSAGE then
    resumeSendingVideo()
  elseif messageType == PICK_PHOTO_MESSAGE then
    local count = t[COUNT]
    onPeerPickingPhoto(count) 
  elseif messageType == ADD_PHOTO_LIST_MESSAGE then
    local photos = t[PHOTO_ID_LIST]
    onPeerAddPhotoList(photos) 
  elseif messageType == POINTER_MESSAGE then
    onPeerPointerDisplay(t)
  elseif messageType == CANCEL_PICKING_PHOTO_MESSAGE then
    onPeerCancelPickingPhoto()      
  elseif messageType == REQUEST_PHOTO_LIST_MESSAGE then
    onPeerRequestPhotoList()        
  else
    local peer_photo = t
    local id = peer_photo[ID] or ""
    local url = peer_photo[URL] or ""
    if messageType == ADD_PHOTO_MESSAGE then
      onPeerAddPhoto(id, url)
    elseif messageType == UPDATE_PHOTO_MESSAGE then
      onPeerUpdatePhoto(id, url)
    elseif messageType == SCROLL_PHOTO_MESSAGE then
      local peerScrollId = peer_photo[SCROLL_ID]
      onPeerScrollPhoto(id, peerScrollId)
    elseif messageType == ACK_MESSAGE then
      onPeerAckPhoto(id)
    else
      print("ERORR: UNKNOWN message type " .. messageType)
    end
  end
end

function onPeerPickingPhoto(count)
  print("onPeerPickingPhoto() count: " .. count)
  if CurrentPhotoCount == 0 then
    uiUpdateLableOnIntroView(count)
  else
    uiDisplayTextIndicator(getPeerPickingPhotoText(count))
    TextIndicator.dismissOnTouchDown = false
    PeerSelectingPhotoCount = count
  end    
  IsPeerPickingPhoto = true
end

function onPeerCancelPickingPhoto()
  IsPeerPickingPhoto = false
  if CurrentPhotoCount == 0 then
    uiUpdateLableOnIntroView(0)
  else  
    uiHideTextIndicator()
  end
end

function onPeerAddPhotoList(photos)
  print("onPeerAddPhotoList()")
  assert(type(photos) == "table")
  
  if IsLowSpeedDevice then
    print ("onImagePickerReturned() we are on a low speed device, stop sending video to save the bandwidth for photo")
    stopSendingVideo()
    p2pSendMessage(STOP_SEND_VIDEO_MESSAGE)
  end
  
  for i, id in pairs(photos) do
    local url = ""
    onPeerAddPhoto(id, url)
  end
   
end  

function onPeerAddPhoto(id, url)
  print("onPeerAddPhoto() id:" .. id .. " url:" .. url)
  if (findPhotoById(id) ~= nil) then
    print("onPeerAddPhoto() duplicated id:" .. id .. ", ignore it.")
    return
  end
  LogViewedAllPhotos = false
  PhotoCountToMe = PhotoCountToMe + 1
  CurrentPhotoCount = CurrentPhotoCount + 1
  IsPeerPickingPhoto = false
  uiHideTextIndicator()
   
  local photo = {}
  photo[ID] = id
  photo[URL] = url
  photo[STATUS] = STATUS_IDLE
  photo[DIRECTION] = DIRECTION_RECEIVER
  photo[PATH] = ""
  photo[PAGE] = CurrentPhotoCount
  photo[IMAGE_SPRITE] = nil
  photo[LOADING_VIEW] = nil
  photo[PHOTO_LOADED] = false
  table.insert(PhotoTable, photo)
  
  if CurrentPage == 0 then
    CurrentPage = 1
    uiUpdatePhotoStatus()
  end
  
  uiCreatePhotoContainer(photo)
  
  downloadNextPhoto()
end

function onPeerUpdatePhoto(id, url)
  assert(not isEmpty(id))
  print("onPeerUpdatePhoto() id:" .. id .. " url:" .. url)
  if isEmpty(url) then
    print("WARN: onPeerUpdatePhoto() url is empty");
    return
  end
  local photo = findPhotoById(id)
  assert(photo)
  if photo then
    photo[URL] = url
    downloadNextPhoto()  
  end
end

function onPeerScrollPhoto(id, peerScrollId)
  assert(not isEmpty(id) and not isEmpty(peerScrollId))
  print("onPeerScrollPhoto() id:" .. id .. "  peerScrollId:" .. peerScrollId)
  
  -- do not update the page if two sides scrolls at the same time, always listen to the session initiator
  if isHost() and peerScrollId ~= CurrentScrollId + 1 then
    print("onPeerScrollPhoto() the other side is not synchronized with the session initiator, session initiator scroll id " .. CurrentScrollId .. ", peer scroll id " .. peerScrollId .. ", send scroll message again.")
    onUserScrollPhoto(CurrentPage) 
    return
  end
  
  CurrentScrollId = peerScrollId  
  
  if id == ID_FOR_SHARE_MORE_PAGE then
    -- NO SHARE MORE PAGE
    return
  else
    local photo = findPhotoById(id)
    if photo then
      CurrentPage = photo[PAGE]
      if CurrentPage == CurrentPhotoCount then
        LogViewedAllPhotos = true
      end
    end 
  end
  PointerSprite:stopAnimation()
  uiScrollToPage(CurrentPage)
  coroutine:yield()
  uiUpdateAdjacentPhotos()
  uiUpdatePhotoStatus()  
end

function onPeerAckPhoto(id)
  assert(not isEmpty(id))
  print("onPeerAckPhoto() id:" .. id)
  local photo = findPhotoById(id)
  assert(photo)
  if photo then
    photo[STATUS] = STATUS_RECEIVED
    if isEveryPhotoReceived() then
      print("onPeerAckPhoto() resume sending video since every photo has been received")
      AvgTransferTime = 0
      resumeSendingVideo()
    end
    uiUpdatePhotoStatus()
  end 
end  

function onPeerPointerDisplay(message)
  assert(not isEmpty(message[ID]))
  print("onPeerPointerDisplay() " .. message[ID])

  -- Not used, might be useful to display based on photo
  local photo = findPhotoById(message[ID])
  if not photo then
    return
  end  

  printf("Display at px=%s py=%s", message[PX], message[PY])
  local width, height = photo[IMAGE_SPRITE]:getSize()
  local left, top = photo[IMAGE_SPRITE]:getPos()  
  local px, py = message[PX], message[PY]
  local x, y = px * width + left, py * height + top
  PointerSprite:playAnimation(x, y)
end  

function onPeerRequestPhotoList()
  print("onPeerRequestPhotoList()")
  
  -- step 1: send photo ids, becuase ADD_PHOTO_LIST_MESSAGE only accepts photo id
  local photoIds = {}
  local count = 0
  for i, photo in ipairs(PhotoTable) do
    if photo and photo[DIRECTION] == DIRECTION_SENDER then
      table.insert(photoIds, photo[ID])
      count = count + 1
    end
  end
  
  if count > 0 then
    p2pSendMessage(ADD_PHOTO_LIST_MESSAGE, photoIds)
  else
    print("onPeerRequestPhotoList() no photo is shared now.")
    return  
  end  
  
  -- step 2: synchronize the current page
  local photo = findPhotoByPage(CurrentPage)
  if photo then
    p2pSendMessage(SCROLL_PHOTO_MESSAGE, photo)
  end  
    
  -- step 3: update photo url
  for i, photo in ipairs(PhotoTable) do
    if photo and photo[DIRECTION] == DIRECTION_SENDER and not isEmpty(photo[URL]) then
      p2pSendMessage(UPDATE_PHOTO_MESSAGE, photo)
    end
  end    
end

------------------------
-- Upload Download management
------------------------

function onUploadFinished (id, success, filePath, url)
  print("UploadFinishedCallback() ENTER id = " .. id .. ", success=" .. tostring(success) .. ", path=" .. filePath .. ", url=" .. url)
  local photo = findPhotoById(id)
  assert(photo)
  if success then 
    photo[PATH] = filePath -- the file path may have been changed(ie, put file into cache)
    photo[URL] = url
    photo[STATUS] = STATUS_SENT
    updateAvgTransferTime(os.time() - photo[TRANSFER_START_TIME])
    p2pSendMessage(UPDATE_PHOTO_MESSAGE, photo)
  else
    photo[STATUS] = STATUS_FAIL_TO_SEND
  end
  
  CurrentUploadToken = CurrentUploadToken + 1
  print("UploadFinishedCallback() increase token count to " .. CurrentUploadToken)
  uploadNextPhoto()
end
  
function onDownloadFinished(id, success, filePath, url)
  print("DownloadFinishedCallback: id = " .. id .. ", success=" .. tostring(success) .. " path=" .. filePath .. " url=" .. url)
  local photo = findPhotoById(id)
  assert(photo)
  if success then
    photo[PATH] = filePath
    photo[STATUS] = STATUS_RECEIVED
    updateAvgTransferTime(os.time() - photo[TRANSFER_START_TIME])
    p2pSendMessage(ACK_MESSAGE, photo)
    uiUpdatePhoto(photo)
  else
    photo[STATUS] = STATUS_FAIL_TO_RECEIVE  
  end
  CurrentDownloadToken = CurrentDownloadToken + 1
  downloadNextPhoto()
end

function uploadNextPhoto()
  assert(CurrentUploadToken >= 0)
  for offset = 0, CurrentPhotoCount - 1 do
    if CurrentUploadToken == 0 then 
      print("uploadNextPhoto() not enough token, upload later")
      break;
    end  
    local page = CurrentPage + offset   -- page starts from 1
    if page >  CurrentPhotoCount then
      page = page - CurrentPhotoCount
    end
    local photo = findPhotoByPage(page)
    assert (photo)
    if photo and photo[DIRECTION] == DIRECTION_SENDER and photo[STATUS] == STATUS_IDLE and not isEmpty(photo[PATH]) then
      local uploading = tango.content.FileTransferService_getService():upload(
          photo[ID],
          tango.content.FileTransferService_IMAGE_FILE, 
          tango.content.FileTransferService_IN_CALL_PHOTO_SHARING,
          photo[PATH])
      if uploading then
        photo[STATUS] = STATUS_SENDING
        photo[TRANSFER_START_TIME] = os.time()
        CurrentUploadToken = CurrentUploadToken - 1
        print("uploadNextPhoto() uploading for page " .. page .. " from " .. photo[PATH] .. ", " .. CurrentUploadToken .. " more tokens left") 
      end
    end
  end
end  

function downloadNextPhoto()
  assert(CurrentDownloadToken >= 0)
  for offset = 0, CurrentPhotoCount - 1 do
    if CurrentDownloadToken == 0 then 
      print("downloadNextPhoto() not enough token, download later")
      break;
    end  
    local page = CurrentPage + offset   -- page starts from 1
    if page >  CurrentPhotoCount then
      page = page - CurrentPhotoCount
    end
    local photo = findPhotoByPage(page)
    assert (photo)
    if photo and photo[DIRECTION] == DIRECTION_RECEIVER and photo[STATUS] == STATUS_IDLE and (not isEmpty(photo[URL])) then
      local downloading = tango.content.FileTransferService_getService():download(
        photo[ID],
        tango.content.FileTransferService_IMAGE_FILE, 
        tango.content.FileTransferService_IN_CALL_PHOTO_SHARING,   
        photo[URL])
      if downloading then
        photo[STATUS] = STATUS_RECEIVING
        photo[TRANSFER_START_TIME] = os.time()
        CurrentDownloadToken = CurrentDownloadToken - 1
        print("downloadNextPhoto() downloading for page " .. page .. " from " .. photo[URL] .. ", " .. CurrentDownloadToken .. " more tokens left") 
      else
        print("ERROR: failed to start downloading " .. photoToString(photo))  
      end
    end
  end
end  

function setupTransferListener()
  print("setupTransferListener()")
  local service = tango.content.FileTransferService_getService()
  if service ~= nil then
    TransferListener = tango.content.LuaFileTransferListener()
    TransferListener:setOnDownloadFinishedCallback(onDownloadFinished)
    TransferListener:setOnUploadFinishedCallback(onUploadFinished)
    service:setTransferListener(TransferListener)
  end
end

function cleanupTransferListener()
  print("cleanupTransferListener()")
  local service = tango.content.FileTransferService_getService() 
  if service ~= nil then
    -- clear the pending upload/download requests
    service:clearPendingRequests();
    -- reset the upload/download listener 
    TransferListener = nil
    service:setTransferListener(TransferListener)
  end
end  

function updateAvgTransferTime(time)
  if isEveryPhotoReceived() then
    print("updateAvgTransferTime() resume sending video since every photo has been received")
    AvgTransferTime = 0
    resumeSendingVideo()
  else
    AvgTransferTime = 0.5 * AvgTransferTime + 0.5 * time
    print("updateAvgTransferTime() AvgTransferTime:" .. AvgTransferTime .. " time:" .. time)
    if AvgTransferTime > PHOTO_TRANSFER_THRESHOLD then
      print("updateAvgTransferTime() AvgTransferTime:" .. AvgTransferTime .. " greater than PHOTO_TRANSFER_THRESHOLD " .. PHOTO_TRANSFER_THRESHOLD .. ", stop sending and receiving video.")
      stopSendingVideo()
      p2pSendMessage(STOP_SEND_VIDEO_MESSAGE)
    end  
  end  
end  

function getPhotoSuccessReceivedCount()
  local ret = 0
  for i, photo in ipairs(PhotoTable) do
    if photo[STATUS] == STATUS_RECEIVED then
      ret = ret + 1
    end  
  end
  return ret 
end  

function isEveryPhotoReceived()
  for i, photo in ipairs(PhotoTable) do
    if photo[STATUS] ~= STATUS_RECEIVED then
      return false
    end  
  end
  return true  
end

------------------------
-- Setup Image Picker
------------------------
function setupImagePicker()
  MOAIImagePicker.setOutputDir(AbsoluteImageDirPath)
  MOAIImagePicker.setScaleParam(MAX_IMAGE_LENGTH, JPEG_COMPRESS_RATIO)
  
  MOAIImagePicker.setListener ( MOAIImagePicker.MULTIIMAGEPICKER_PICKING,
    onUserPickingPhoto
  )
  
  local function onImagePickerReturned(count)
    print ("onImagePickerReturned() image picker returned " .. count .. " images " )
    ImagePickerVisible = false
    MOAIImagePicker.dismissPicker();
    
    if count == 0 then
      if ShouldExitIfNoPhotoPicked then
        print("user didn't pick any photo, return directly")
        local function exitLater()
          coroutine.yield();
          userExit()
        end  
        local exitHandler = MOAICoroutine.new()
        exitHandler:run ( exitLater)
      end 
      p2pSendMessage(CANCEL_PICKING_PHOTO_MESSAGE) 
      return
    end
    
    ShouldExitIfNoPhotoPicked = false
    if not isUIInitialized() then
      createUIElements()
    end  
    
    local dstPage = CurrentPhotoCount + 1
    local reachedCapacity = false
    local photos = {}
    
    for i = 1, count do
      local id = PeerId .. "_" .. CurrentPhotoId
      local photo = onUserAddPhoto(id, nil, nil)
      table.insert(photos, photo[ID])
      CurrentPhotoId = CurrentPhotoId + 1
      if CurrentPhotoCount >= MAX_PHOTO_COUNT then
        print("onImagePickerReturned() MAX_PHOTO_COUNT " .. MAX_PHOTO_COUNT .. " reached, do not add more photos")
        reachedCapacity = true
        break
      end
    end
    
    -- inform peer
    p2pSendMessage(ADD_PHOTO_LIST_MESSAGE, photos)

    -- Scroll to the the first new image
    PhotoScroller:updateLayout()
    uiScrollToPage(dstPage)
    onUserScrollPhoto(dstPage)
    
    if IsLowSpeedDevice then
      print ("onImagePickerReturned() we are on a low speed device, stop sending video to save the bandwidth for photo")
      stopSendingVideo()
      p2pSendMessage(STOP_SEND_VIDEO_MESSAGE)
    elseif OSType == OS_ANDROID then
      resumeSendingVideo()
    end
    
    if reachedCapacity then
      uiDisplayTextIndicator(localize("sp_warning_can_not_add_more") .. " " .. MAX_PHOTO_COUNT .. " " .. localize("sp_photos"))
      TextIndicator.dismissOnTouchDown = true
    end
  end
  
  MOAIImagePicker.setListener ( MOAIImagePicker.MULTIIMAGEPICKER_FINISHPICKING,
    onImagePickerReturned
  )
  
  local function onImageGenerated(dict)
      print ("onImageGenerated() get photos from image picker ")
      
      assert(type(dict) == "table")
      local fileName = dict["FILE_NAME"]
      assert(fileName ~= nil)
      local fullPath = AbsoluteImageDirPath .. "/" .. fileName
      local imageData = dict["DATA"]
      
      local photo = getNextEmptyPhoto(DIRECTION_SENDER)
      if photo == nil then
        print("onImageGenerated() can not find a photo object for this image, return")
        return
      end  
      
      if (DEBUG_USE_DEFAULT_IMAGE or MOAIFileSystem.checkFileExists(fullPath)) then 
        onUserPhotoGenerated(photo, fullPath, imageData)
      else
        print ("file " .. fullPath .. " does not exist")  
      end
     
      print ("onImageGenerated() EXIT")
  end
    
  MOAIImagePicker.setListener ( MOAIImagePicker.MULTIIMAGEPICKER_GENERATEIMAGE,
    function (data)
      --Executors.callLaterTime(0.3, processPhotos, data)
      onImageGenerated(data)
    end
  )

  MOAIImagePicker.setListener ( MOAIImagePicker.MULTIIMAGEPICKER_DIDCANCEL,
    function ()
      onImagePickerReturned(0)
    end
  )
end

------------------------
-- MOAIP2PChannel
------------------------
MOAIP2PChannel.setOnStateChangedCallback(
  function(newState)
    if newState == MOAIP2PChannel.CONNECTED then
      setupPeerInfo()
    end
    print ('State notification received: ' .. newState .. ", PeerId: " .. PeerId)
  end
)

MOAIP2PChannel.setOnReceiveCallback(
  function()
    print ('Got receive notification')
    local data = nil
    local currState = MOAIP2PChannel.getState()
    if currState == MOAIP2PChannel.CONNECTED then
      data = MOAIP2PChannel.receive()
    else
      print ('Channel disconnected, can not get data now')
    end

    if data ~= nil then
      print("received new data " .. data)
      local t = MOAIJsonParser.decode(data)
      if (t) then
        local incomingMessageHandler = MOAICoroutine.new()
        incomingMessageHandler:run ( onPeerReceiveMessage, t )
      else
        print("ERROR: can not decode json from " .. data)
      end      
    end
  end
)


function p2pSendMessage(messageType, object)
  print('p2pSendMessage: ' .. messageType)
  local t= {}
  t[MESSAGE_TYPE] = messageType
  t[PROTOCOL_VERSION] = CURRENT_PROTOCOL_VERSION
  if messageType == STOP_SEND_VIDEO_MESSAGE or messageType == RESUME_SEND_VIDEO_MESSAGE  or messageType == CANCEL_PICKING_PHOTO_MESSAGE or messageType == REQUEST_PHOTO_LIST_MESSAGE then
    -- nothing
  elseif messageType == PICK_PHOTO_MESSAGE then
    local count = object
    t[COUNT] = count
  elseif messageType == ADD_PHOTO_LIST_MESSAGE then
    assert(type(object) == "table")
    local photos = object
    t[PHOTO_ID_LIST] = photos
  else
    local photo = object
    t[ID] = photo[ID] or ""
    t[URL] = photo[URL] or ""
    if messageType == SCROLL_PHOTO_MESSAGE then  
      t[SCROLL_ID] = CurrentScrollId
    end  
  end
  
  p2pSendRawMessage(t)
end
  
function p2pSendRawMessage(t)
  local msg = MOAIJsonParser.encode(t)
  local st = MOAIP2PChannel.getState()

  print("p2p state = " .. st)
  if st == MOAIP2PChannel.CONNECTED then
    print('Sending: ' .. msg .. "  LENGTH:" .. string.len(msg))
    MOAIP2PChannel.send( msg )
  else
    print('network is not available when sending ' .. msg)
  end
end

function setupPeerInfo()
  local st = MOAIP2PChannel.getState()
  if st == MOAIP2PChannel.CONNECTED then
    PeerId = MOAIP2PChannel.getPeerId()
    Myself = TANGOAppContext.getMyInfo()
    local peers = TANGOAppContext.getParticipantsInfo()
    if peers ~= nil then
     for id, value in pairs(peers) do
       PeerInfo = value
       break
     end   
    end  
    print("Infomation about myself:")
    printTable(Myself)
    print("isHost " .. toString(isHost()))
    print("Infomation about peer:")
    printTable(PeerInfo)
  end
  print ('setupPeerInfo: distinct peer id is ' .. PeerId)
end
------------------------
-- File management
------------------------
function setupOutputDir()
  print("setupOutputDir() " .. AbsoluteImageDirPath)
  MOAIFileSystem.affirmPath(AbsoluteImageDirPath)
end

function cleanUpOutputDir()
  print("cleanUpOutputDir() " .. AbsoluteImageDirPath)
  MOAIFileSystem.deleteDirectory(ImageDirName, true)
end

------------------------
-- Metadata Manipulation
------------------------

function findPhotoById(id)
  for i, photo in ipairs(PhotoTable) do
    if photo and photo[ID] == id then
      return photo
    end
  end
  print("ERORR: can not find photo with id " .. id) 
  return nil    
end

function findPhotoByPage(page)
  local ret = PhotoTable[page]
  if ret == nil then
    print("ERORR: can not find photo with page " .. page)
  end     
  return ret    
end

function getNextEmptyPhoto(direction)
  for i, photo in ipairs(PhotoTable) do
    if photo and photo[DIRECTION] == direction and isEmpty(photo[PATH]) then
      return photo
    end
  end
  print("ERORR: can not find empty photo with direction " .. direction) 
  return nil  
end  

function photoToString(photo)
  local str = "Photo page:" .. photo[PAGE]  .. " direction:" .. photo[DIRECTION] .. " status:" .. photo[STATUS] .. " id:" .. photo[ID] .. "path:" .. photo[PATH] .. " url:" .. photo[URL] 
  return str
end


-- in some extreme cases, both sides could be initiator
function isInitiator()
  return DEBUG_FORCE_AS_INITIATOR or Myself and Myself.isInitiator or Myself == nil
end  

-- this is unique on both sides
function isHost()
  return DEBUG_FORCE_AS_INITIATOR or MOAIP2PChannel and MOAIP2PChannel.getPeerId() == 0
end  

------------------------
-- initilization the UI elements
------------------------
function onCreate(params)
    print ("onCreate()")
    
    setup()
    
    PageView = View {
        scene = scene,
    }
    
    AdjustedView = AdjustedView {}
    AdjustedView:setView(PageView)
    
    PipLayer = Layer {scene = scene}
    PipLayer:setSortMode(MOAILayer.SORT_PRIORITY_ASCENDING)
    PipLayer:setTouchEnabled(true)
    PipLayer:setSize(Application.screenWidth, Application.screenHeight)
    
    if isInitiator() and OSType == OS_ANDROID then
      ShouldShowPopup = true
      ShouldExitIfNoPhotoPicked = true
      onPickPhoto()
      return
    end  
    
    createUIElements()
    
    if not isInitiator() then
      p2pSendMessage(REQUEST_PHOTO_LIST_MESSAGE)
    end  
end
    
function createUIElements()
    print ("createUIElements() ENTER")
    local width, height = getAdjustedViewSize()
    local originX, originY = getAdjustedPos(0, 0)
    
    -- initialize IntroView
    IntroView = nil
    if CurrentPhotoCount == 0 then
      IntroView = UIIntroView {
        parent = PageView,
        size = { width, height }, 
        peerName = getPeerName(),
        isInitiator = isInitiator(),
        isSlowDevice = IsLowSpeedDevice,
      }
      if IntroView.shareButton then
        IntroView.shareButton:setOnClick(onPickPhoto)
      end  
    end

    -- initialize PhotoScroller
    PhotoScroller = Scroller {
        parent = PageView,
        adjustedView = AdjustedView,
        layout = HBoxLayout {
            align = {"center", "center"},
            padding = {0, 0, 0, 0},
            gap = {50, 0},
        },
        vScrollEnabled = false
    }
    PhotoScroller:setPos(getAdjustedPos(0, 0))
    PhotoScroller:setEventListener(PhotoScroller.EVENT_PAGE_WILL_CHANGE, onScrollBegin)    
    PhotoScroller:setEventListener(PhotoScroller.EVENT_PAGE_CHANGED, onScrollEnded)    
    PhotoScroller:setEventListener(PhotoScroller.EVENT_TAP, uiScrollTapEvent)
    PhotoScroller:setEventListener(PhotoScroller.EVENT_TOUCH_DOWN, onTouchDownScroller)
    
    -- add photos to scroller
    if CurrentPhotoCount > 0 then
      print("createUIElements() Start adding containers for photos")
      for i, photo in ipairs(PhotoTable) do
        uiCreatePhotoContainer(photo) 
      end
      print("createUIElements() End adding containers for photos")
            
      uiUpdateAdjacentPhotos()      
    end

    -- initialize status bar
    ProgressBar = UIProgressBar {
      size = { 0, 6},
      pos = { getAdjustedPos(0 , 0) },
      parent = PageView
    }
    
    -- initialize buttons
    ExitIcon = setupExitIcon(AdjustedView)
    ExitIcon:setOnClick(onExitIconClick)
    AddMoreIcon = setupAddMoreIcon(AdjustedView)
    AddMoreIcon:setOnClick(onShareMorePhoto)
    if isInitiator() then
      AddMoreIcon:setColor(0, 0, 0, 0)
    end
    
    -- initialize status lable
    PageNumberLabelContainer = Button {
      name = "PageNumber",
      parent = PageView,
      size = { 140, 68},
      styles = {PAGE_NUMBER_STYLES}
    }
    PageNumberLabelContainer:setPos(getAdjustedPos( (width - 140)/2, height - 34 - 68))
    PageNumberLabelContainer:setColor(0, 0, 0, 0)

    
    -- initialize pointer    
    PointerSprite = Sprite {
      texture = "../assets/target_pointer.png",
      touchEnabled = false
    }    
    
    function PointerSprite:playAnimation(px,py)
      if not isOnPhotoPage() then
        return
      end
      print("Play Pointer Animation")
      self:stopAnimation()
      self:setColor(1, 1, 0, 0.7)            
      self:setParent(PageView)  
      self:setCenterPos(getAdjustedPos(px, py))  
      self.animation = Animation({self}):fadeIn():loop(1, Animation({self}, 0.25)
                                              :setScl(0.75, 0.75)
                                              :moveScl(0.15, 0.15)
                                              :moveScl(-0.15, -0.15))
                                              :fadeOut()
                                              :setVisible(false)
                                              
      local completeHandler = function(animation)
        print("pointer animation complete!")     
        self:setParent(nil)
      end
      self.animation:play({onComplete = completeHandler})                                        
    end
    
    function PointerSprite:stopAnimation()
      if(self.animation) then
        self.animation:stop()
        self:setParent(nil)
      end
    end
    
    -- initialize pip and mute button 
    setupMuteButton(AdjustedView)
    if VideoView ~= nil then
      VideoView:dispose()
    end  
    VideoView = setupVideoView(AdjustedView, PipLayer)   
    
    
    -- draw the IntroView based on the position of VideoView
    if IntroView then
      IntroView:setPos(getAdjustedPos(0, 0))
      IntroView:setBottomLine(VideoView ~= nil and (CurrentOrientation == Orientation.PORTRAIT_UP or CurrentOrientation == nil) and ((VideoView:getTop() - 20) * (TARGET_SCREEN_HEIGHT / Application.screenHeight)) or height)
      IntroView:resizeHandler()
      if not isInitiator() then
        uiUpdateLableOnIntroView(0)
      end
    end  
    
    ExitDialog = nil  
    TextIndicator = nil
    
    FlagFirstEnterScreen = false
    print ("createUIElements() EXIT")
end

function onExitIconClick(e)
    print("onExitIconClick")
    uiDisplayExitDialog()
end

function onPickPhoto(e)
    print("onPickPhoto ")
    if OSType == OS_ANDROID then
      stopSendingVideo()
    end
    if MOAIImagePicker.setMaxImageCount then
      MOAIImagePicker.setMaxImageCount(MAX_PHOTO_COUNT - CurrentPhotoCount)
    end
    if MOAIImagePicker.setShowPopUpOverlay then
      MOAIImagePicker.setShowPopUpOverlay(ShouldShowPopup)
      ShouldShowPopup = false -- only show it once
    end  
    onUserPickingPhoto(0)
    MOAIImagePicker.displayAlbumPicker ()
    ImagePickerVisible = true
    LogClickPickPhotoButton = true
end

function onShareMorePhoto()
  onPickPhoto()
  LogBeenShareMorePhoto = true
end  

function onScrollBegin(event)
    onUserScrollPhotoBegin(event.data + 1)
end

function onScrollEnded(event)
    -- the page in this file starts from 1, but for PhotoScroller, from 0
    onUserScrollPhotoEnded(event.data + 1)
end

function onUserPressBackButton ()
	print ( "User press the back button on android" )
  if ExitDialog == nil then
    uiDisplayExitDialog()
  else
    uiDismissExitDialog()
  end    
	return true
end

function onTouchDownScroller(e)
  if TextIndicator ~= nil and TextIndicator.dismissOnTouchDown then
    uiHideTextIndicator()
  end  
end  

function userExit()
  LogExitBy = "me"
  MOAISim.exit()
end

------------------------
-- Change the UI elements
------------------------

local function getImageTexture(photo, imageData)
  local img = nil;
  local imagePath = photo[PATH] or ""
  if isEmpty(imagePath) or DEBUG_USE_DEFAULT_IMAGE then
    print("image not available")
    img = nil
  elseif imageData == nil then  
    print("load from file " .. imagePath)
    img = MOAIImageTexture.new ()
    img:load(imagePath)
  else
    print("load from data directly")
    local dataBuf = MOAIDataBuffer.new ()
    dataBuf:setString(imageData)
    img = MOAIImageTexture.new ()
    img:loadFromBuffer(dataBuf)
  end
  return img
end

-- compute the width and height of the img view
local function getImageShowSize(imgWidth, imgHeight)
  --print ( (imgWidth >= imgHeight and "Horizontal" or "Vertical") .. " width:" .. imgWidth .. " height:" .. imgHeight)
  local width, height = imgWidth, imgHeight
  local viewWidth, viewHeight = getAdjustedViewSize()
  
  -- first try scale down according to width
  width = viewWidth
  height = imgHeight * (width/imgWidth)
  if height > viewHeight then
    -- scale down according to height
    height = viewHeight
    width = imgWidth * (height/imgHeight)
    assert(width <= viewWidth)
  end  
  
  return width, height
end  

function isEmptyPhotoContainer(photo)
  return photo[IMAGE_CONTAINER] == nil or photo[IMAGE_SPRITE] == nil
end  

function uiCreatePhotoContainer(photo)
  if IntroView ~= nil then
    print("Memory Usage before clearing IntroView: ")
    printMemoryUsage()
        
    IntroView:clearTexture()    
    IntroView:recursivelyRemoveChildren()
    IntroView:dispose()
    IntroView = nil
    
    print("Memory Usage after clearing IntroView: ")
    printMemoryUsage()
  end
  
  local container = View {
    parent = PhotoScroller,
    size = { getAdjustedViewSize() },    
    name = IMAGE_CONTAINER
  }
  photo[IMAGE_CONTAINER] = container
  photo[IMAGE_SPRITE] = nil
  photo[LOADING_VIEW] = nil
  photo[PHOTO_LOADED] = false
end  

function uiClearPhotoContainer(photo)
  photo[PHOTO_LOADED] = false
  
  if isEmptyPhotoContainer(photo) then
    return
  end  
  
  local container = photo[IMAGE_CONTAINER]  
  print ("uiClearPhotoContainer() for page " .. photo[PAGE])
  container:recursivelyRemoveChildren()
    
  local loadingView = photo[LOADING_VIEW]
  if loadingView ~= nil then
    loadingView:stopAnimation()
    loadingView:setParent(nil)
    photo[LOADING_VIEW] = nil
  end
  
  photo[IMAGE_SPRITE] = nil
end  

-- create the stucture inside the photo container, but do not fill the sprite with real photo data
function uiFillPhotoContainer(photo)
  if not isEmptyPhotoContainer(photo) then
    return
  end
  local container = photo[IMAGE_CONTAINER]
  local imagePath = photo[PATH] or ""
  print ("uiFillPhotoContainer() for page " .. photo[PAGE])
  
  local subContainer = View {
    parent = container,
    size = { getAdjustedViewSize() },
    name = IMAGE_SUB_CONTAINER,
    layout = BoxLayout {
      align = {BoxLayout.HORIZOTAL_CENTER, BoxLayout.VERTICAL_CENTER},
    }
  }
  
  local loadingView = UILoadingView {
    size = { getAdjustedViewSize() },            
  }
  photo[LOADING_VIEW] = loadingView
  
  local sprite = Sprite {
    parent = subContainer,
    texture = nil,
    size = { 0, 0 },
    name = IMAGE_SPRITE
  } 
  photo[IMAGE_SPRITE] = sprite
  
  PhotoScroller:updateLayout()
end

-- if photo data is available, update with real photo data, or else show the loading view
function uiUpdatePhoto(photo, imageData)
  assert(photo)
  if not isInLoadArea(photo[PAGE]) then
    print("uiUpdatePhoto() for page " .. photo[PAGE] .. " is not in the loading area, do not update photo now.")
    return
  end
  
  if photo[PHOTO_LOADED] then
    -- view already loaded with picture data
    return
  end  
  
  if photo[IMAGE_SPRITE] == nil then
    return
  end  
  
  local loadingView = photo[LOADING_VIEW]
  
  local img =  getImageTexture(photo, imageData)
  if img then
    print("uiUpdatePhoto() for page " .. photo[PAGE] .. ", update with real photo.")
    photo[PHOTO_LOADED] = true
    -- remove the loading view
    loadingView:stopAnimation()
    loadingView:setParent(nil) 
    photo[LOADING_VIEW] = nil
        
    -- set picture texture    
    photo[IMAGE_SPRITE]:setTexture(img)
    local width, height = getImageShowSize(img:getSize())
    photo[IMAGE_SPRITE]:setSize(width, height)
    photo[IMAGE_SPRITE]:setCenterPos(photo[IMAGE_SPRITE]:getParent():getCenterPos())
    photo[IMAGE_SPRITE]:getParent():updateComponent()

  else
    print("uiUpdatePhoto() for page " .. photo[PAGE] .. ", show loading view.")
    -- show loading view
    loadingView:startAnimation()
    local subContainer = photo[IMAGE_CONTAINER]:getChildByName(IMAGE_SUB_CONTAINER)
    assert(subContainer)
    loadingView:setParent(subContainer)
  end
  
end

function isInLoadArea(page)
  return ( CurrentPhotoCount <= (RELOAD_PHOTO_SPAN * 2 + 1) ) or ( page >= (CurrentPage - RELOAD_PHOTO_SPAN) and (page <= CurrentPage + RELOAD_PHOTO_SPAN) )
end  

function uiUpdateAdjacentPhotos()
  print("uiUpdateAdjacentPhotos() current page: " .. CurrentPage)
  for page = 1, CurrentPhotoCount do
    local photo = findPhotoByPage(page)
    assert(photo)
    if isInLoadArea(page) then
      uiFillPhotoContainer(photo, nil)
      uiUpdatePhoto(photo)
    else
      uiClearPhotoContainer(photo)
    end  
  end
  if CurrentScrollId % 5 == 0 then
    MOAISim.forceGarbageCollection()
  end  
  if DEBUG_SHOW_MEMORY_USAGE then
    printMemoryUsage()
  end
end  

function getPeerName()
  return PeerInfo and isAsciiStr(PeerInfo.name) and PeerInfo.name or localize("sp_peer")
end  

function getPeerPickingPhotoText(photoCount)
  local nameStr = getPeerName()
  return nameStr .. " " .. localize("sp_intro_view_receiver_tip") ..  ( (photoCount == 0 and "") or (" (" .. photoCount .. ")") )
end

function uiUpdateLableOnIntroView(photoCount)
  IntroView:setPhotoCount(photoCount)
end  



function uiUpdatePhotoStatus() 
  print("uiUpdatePhotoStatus() ")
  local photo = findPhotoByPage(CurrentPage)  
  if photo == nil then
    print("uiUpdatePhotoStatus() can not find the photo")
    return
  end
  
  local statusVisible = isOnPhotoPage() and photo
  if statusVisible then
    PageNumberLabelContainer:setColor(1,1,1,1)
    local scrollPos = CurrentPage .. " " .. localize("sp_scroll_page_of") .. " " .. CurrentPhotoCount
    PageNumberLabelContainer:setText(scrollPos) 
  else
    PageNumberLabelContainer:setColor(0,0,0,0)
  end
  
  local progressBarVisible = isOnPhotoPage()
                             and photo 
                             and photo[DIRECTION] == DIRECTION_SENDER 
                             and photo[STATUS] ~= STATUS_RECEIVED
  if progressBarVisible then
    ProgressBar:setVisible(true)
    if ProgressBar:getWidth() == 0 then
      ProgressBar:setWidth(getAdjustedViewWidth())
      ProgressBar:resizeHandler()
    end  
    ProgressBar:startAnimation()
  else
    ProgressBar:setVisible(false)
  end  
  
  local scrollIndicator = CurrentPage == 1 and CurrentPhotoCount > 1 and not FlagShowedScrollerIndicatorOnFirstPage and isInitiator()
  local peerPickingIndication = CurrentPhotoCount > 0 and  IsPeerPickingPhoto
  local TextIndicatorVisible =  isOnPhotoPage() and (scrollIndicator or peerPickingIndication)
  if TextIndicatorVisible then
    if scrollIndicator then
      uiDisplayTextIndicator(localize("sp_scroll_indicator"))
      TextIndicator.dismissOnTouchDown = true
    else
      uiDisplayTextIndicator(localize(getPeerPickingPhotoText(PeerSelectingPhotoCount)))
      TextIndicator.dismissOnTouchDown = false
    end
  else  
    uiHideTextIndicator()
  end
  
  if CurrentPhotoCount < MAX_PHOTO_COUNT then
    AddMoreIcon:setStyles(CurrentPage == CurrentPhotoCount and ADD_MORE_ICON_HIGHLIGHTED_STYLES or ADD_MORE_ICON_NORMAL_STYLES)
    AddMoreIcon:setColor(1, 1, 1, 1)
  else
    AddMoreIcon:setColor(0, 0, 0, 0)
  end    
  AddMoreIcon:updateLayout()
end  

function uiDisplayExitDialog()
  if ExitDialog ~= nil then
    return
  end  
  FlagDisplayExitDialog = true
  local width, height = getAdjustedViewSize()
  local smaller = math.min(width, height)
  ExitDialog = UISimpleDialog {
    size = { math.min(750, smaller - 100), 0},
    parent = PageView
  }
  ExitDialog:setText(localize("sp_exit_dialog_desc") , localize("sp_exit_dialog_yes"), localize("sp_exit_dialog_no"))
  ExitDialog:setCenterPos( getAdjustedPos( getAdjustedViewWidth() / 2, getAdjustedViewHeight() / 2 ) )

  ExitDialog:setOnYesButtonClick(userExit)

  ExitDialog:setOnNoButtonClick(uiDismissExitDialog)
  
  uiHideTextIndicator()
  
  PhotoScroller:setTouchEnabled(false)
end

function uiDismissExitDialog()
  if ExitDialog then
    PageView:removeChild(ExitDialog)
    ExitDialog = nil
    PhotoScroller:setTouchEnabled(true)
  end  
  FlagDisplayExitDialog = false
end

function uiDisplayTextIndicator(text)
  if TextIndicator then
    TextIndicator:setText(text)
    return
  end  
  
  TextIndicator = UIRoundedTextLabel {
    bgColor = {0.15, 0.15, 0.15, 0.7},
    parent = PageView,
    size = { math.min (700, getAdjustedViewWidth() - 100), 70 },
    text = text,
    textSize = 30
  }
  TextIndicator:setCenterPos( getAdjustedPos(getAdjustedViewWidth()/2 , getAdjustedViewHeight()/2) )
  TextIndicator:draw()
end  

function uiHideTextIndicator()
  if TextIndicator then
    PageView:removeChild(TextIndicator)
    TextIndicator = nil
    FlagShowedScrollerIndicatorOnFirstPage = true  
  end
end

function isOnPhotoPage()
  return CurrentPhotoCount > 0 and CurrentPage <= CurrentPhotoCount 
end  

function isUIInitialized()
  return PhotoScroller ~= nil
end  

------------------------
-- PhotoScroller Events
------------------------

function uiScrollToPage(page)
  print("uiScrollToPage() " .. page)
  if page <= 0 or page >= CurrentPhotoCount + 1 then
    return
  end  
  PhotoScroller:updateLayout()
  PhotoScroller:setPage(page - 1, 1.0)
end

function uiScrollTapEvent(event)
  print("uiScrollTapEvent")
  local e = event.data
  if not isOnPhotoPage() then
    return
  end
  
  e.x, e.y = AdjustedView:getAdjustedTapPoint(e.x, e.y)
  
  if e.y < 100 then
    return
  end  
  
  local photo = findPhotoByPage(CurrentPage)  
  if not photo then
    return
  end  
  local width, height = photo[IMAGE_SPRITE]:getSize()
  local left, top = photo[IMAGE_SPRITE]:getPos()
  local px = (e.x - left) / width
  local py = (e.y - top)/ height
  
  local message = {
    MESSAGE_TYPE = POINTER_MESSAGE,
    ID = photo[ID],
    PX = px,
    PY = py,
  }
      
  PointerSprite:playAnimation(e.x, e.y)
  
  p2pSendRawMessage(message)
end

------------------------
-- Orientation Events
------------------------

function getAdjustedViewWidth()
  return AdjustedView:getAdjustedViewWidth()
end  

function getAdjustedViewHeight()
  return AdjustedView:getAdjustedViewHeight()
end  

function getAdjustedViewSize()
  return AdjustedView:getAdjustedViewSize()
end  

function getAdjustedPos(x, y)
  return AdjustedView:getAdjustedPos(x, y)
end

function observeOrientation(scene)
  
  if(DEBUG_DISABLE_ORIENTATION) then
    return
  end
  
  if IsLowSpeedDevice then
    print("we are on low speed device, ignore orientation chagnes.")
    return
  end  
  
  local function handleOrientation(orientation, angle)
      if orientation == nil then
        return
      end  
      print("orientation: " .. orientation .. " angle=" .. angle  .. " sw: " .. scene:getWidth() .. " sh: " .. scene:getHeight())
      
      if(CurrentPhotoCount == 0) then
        return
      end
      
      setCurrentOrientation(orientation)
      local vertical = orientation == Orientation.PORTRAIT_UP or orientation == Orientation.PORTRAIT_DOWN
      
      local offsetX, offsetY = 0, 0
      if not vertical then
        local delta = (TARGET_SCREEN_HEIGHT - TARGET_SCREEN_WIDTH) / 2
        offsetX, offsetY =  -delta, delta
      end  
      AdjustedView:setOffset(offsetX, offsetY)
      
      PageView:recursivelyRemoveChildren()
      
      
      print("Memory Usage before garbage collection: ")
      printMemoryUsage()
      
      MOAISim.forceGarbageCollection()
      print("Memory Usage after garbage collection: ")
      printMemoryUsage()
      
      createUIElements()
      
      print("Memory Usage after screen recreated: ")
      printMemoryUsage()
          
      -- rotate the PageView
      PageView:setCenterPiv()
      PageView:updateLayout()   
      local rotAngle = 0
      if(orientation == Orientation.PORTRAIT_UP) then
        rotAngle = 0
      elseif(orientation == Orientation.PORTRAIT_DOWN) then
        rotAngle = 180
      elseif(orientation == Orientation.LANDSCAPE_LEFT) then
        rotAngle = 90
      elseif(orientation == Orientation.LANDSCAPE_RIGHT) then
        rotAngle = -90
      end
      PageView:setRot ( 0 , 0, rotAngle )    
      
      PhotoScroller:updateLayout()
      
      uiScrollToPage(CurrentPage)
      uiUpdatePhotoStatus()
      
      if CurrentPhotoCount == 0 and not isInitiator() then
       uiUpdateLableOnIntroView(0)
      end
      
      if FlagDisplayExitDialog then
        uiDisplayExitDialog()
      end  
  end
  print("setting orientation callback")
  local orientation = DeviceOrientation.new()
  orientation:setCallback(handleOrientation)
  print("set orientation callback")
end

