module(..., package.seeall)

local table             = require "hp/lang/table"
local class             = require "hp/lang/class"
local Event             = require "hp/event/Event"
local Component         = require "hp/gui/Component"
local Animation         = require "hp/display/Animation"

-- class define
local M                 = class(Component)
local super             = Component

local colorDark = {0.190,	0.330,	0.420,	1}
local colorWhite = {1,	1,	1,	1}

function M:initInternal(params)
    super.initInternal(self, params)
end

function M:createChildren()
  
  self.progressBackground = Graphics {
    size = { width, height},
    pos = { 0, 0 },
    parent = self,
    color = self.colorWhite
  }
  self.progressBackground:fillRect()
  
  
  self.bar = Graphics {
    size = { progressBarWidth, height},
    pos = { 0, 0},
    parent = self,
    color = self.colorDark
  }
  self.bar:fillRect()
  
 
end

function M:resizeHandler(e)
  
  local width, height = self:getSize()
  print("progress_bar resizeHandler() width " .. width .. " height " .. height)
  self.progressBackground:setSize(width, height)
  self.progressBarWidth = math.min(width, 50)
  self.bar:setSize(self.progressBarWidth, height)
  
  
  self:stopAnimation()
  self.progressBarAnimation = Animation():loop(0,  Animation():sequence(
       Animation(self.bar, 3, MOAIEaseType.LINEAR):seekScl(width / self.progressBarWidth * 2):setScl(1,1):setColor(table.unpack(colorWhite)),
       Animation(self.progressBackground):setColor(table.unpack(colorDark)),
       Animation(self.bar, 3, MOAIEaseType.LINEAR):seekScl(width / self.progressBarWidth * 2):setScl(1,1):setColor(table.unpack(colorDark)),
       Animation(self.progressBackground):setColor(table.unpack(colorWhite))
       ))
end  

function M:updateLayout()
    super.updateLayout(self)
end

function M:startAnimation()
  if  self.progressBarAnimation:isRunning() then
    return
  else  
    self:resetColorAndWidth()
    self.progressBarAnimation:play()
  end  
end

function M:stopAnimation()
  if self.progressBarAnimation and self.progressBarAnimation:isRunning() then
    self.progressBarAnimation:stop()
  end  
end

function M:setFullProgress()
  self:stopAnimation()
  self:resetColor()
  local width, height = self:getSize()
  self.bar:setSize(width, height)
end

function M:resetColor()
    self.bar:setColor(table.unpack(colorDark))
    self.progressBackground:setColor(table.unpack(colorWhite))
end

function M:resetColorAndWidth()
    self:resizeHandler()
    self:resetColor()
end

return M


