local table             = require "hp/lang/table"
local class             = require "hp/lang/class"
local Event             = require "hp/event/Event"
local Component         = require "hp/gui/Component"
local NinePatch         = require "hp/display/NinePatch"

require "global_variables"


-- class define
local M                 = class(Component)
local super             = Component

function M:initInternal()
  super.initInternal(self)
end

function M:createChildren()
  self._bg = NinePatch {
    texture = "../assets/popup_bg.png", 
    parent = self
  }
  self._bg:setAlpha(0.9)
  
  self._desc = TextLabel {
    text = "-----------------Desc------------------",
    parent = self,
    textSize = 30,
  }
  self._desc:setRGBA(207,207,207,1)
  self._desc:setAlignment(TextLabel.HORIZOTAL_ALIGNS.center, TextLabel.VERTICAL_ALIGNS.center)
  
    
  self._yesButton = Button {
    text = "Yes",
    parent = self,
    size = { 180, 60 },
    styles = { OK_BUTTON_STYLES }
  }
  
  self._noButton = Button {
    text = "No",
    parent = self,
    size = { 180, 60 },
    styles = { CANCEL_BUTTON_STYLES }
  }
end

function M:setText(desc, yesText, noText) 
  self._desc:setText(desc)
  self._yesButton:setText(yesText)
  self._noButton:setText(noText)
end  

function M:setOnYesButtonClick(click)
  self._yesButton:setOnClick(click)
end  

function M:setOnNoButtonClick(click)
  self._noButton:setOnClick(click)
end 

function M:resizeHandler(e)  
  local desc_height = 70
  local padding_left = 34
  self._desc:setSize(self:getWidth()- 2 * padding_left, desc_height)
  self._desc:setPos(padding_left, 36)
  
  local button_space = 16
  local button_width = math.min(200, self:getWidth() / 2 - button_space / 2 - padding_left)
  self._noButton:setWidth (button_width)
  self._yesButton:setWidth (button_width)
  self._noButton:setPos(self:getWidth() / 2  - button_space / 2 - button_width, self._desc:getBottom() + 25)
  self._yesButton:setPos(self:getWidth() / 2 + button_space / 2, self._noButton:getTop())
  
  self:setHeight(self._noButton:getBottom() + 42)
  
  self._bg:setSize(self:getSize())
end

return M