------------------------
-- Static Utilities
------------------------
require "global_variables"

function isEmpty(str)
  return str == nil or str == ""
end  

function printTable(t)
  if t == nil then
    print "table is nil"
    return
  end  
  for i,v in pairs(t) do 
    print(i, v)
  end  
end  

function isAsciiStr(str)
  assert(type(str) == "string")
  for i = 1, string.len(str) do
    if string.byte(str, i) > 127 then
      return false
    end  
  end  
  return true
end  

function printMemoryUsage()
  local memoryTable = MOAISim.getMemoryUsage()
  print("Memory Usage: Total = " .. memoryTable.total/MEGA .. "M; Texture = " .. memoryTable.texture/MEGA .. "M; Lua = " .. memoryTable.lua/MEGA .. "M.")
end  

function printf ( ... )
	return print ( string.format ( ... ))
end 

function getImageCenterRect(width, height, base)
  local min = math.min(width, height)
  
  if min <= base then
    return 0, 0, width, height
  end

  local size = base * (math.floor(min/base))
      
  local xStart = math.floor( (width - size) / 2)
  local yStart = math.floor( (height - size) / 2)
  return xStart, yStart, xStart + size, yStart + size
end  

function toString(a)
  if a == nil then
    return "nil"
  end
    
  local t = type(a)
  if t == "boolean" then
    return a and "true" or "false"
  elseif t == "string" then
    return a
  elseif t == "number" then
    return "" .. a      
  else
    return "type " .. t  
  end
end

function string.starts(String,Start)
  if String == nil or Start == nil then
    return false
  end
  return string.sub(String,1,string.len(Start))==Start
end

function string.ends(String,End)
  if String == nil or End == nil then
    return false
  end
  return End=='' or string.sub(String,-string.len(End))==End
end


-- LOLALIZATION RELATED FUNCTION AND TABLES
local english_string_table = {
  sp_intro_view_button = "Select Photos",
  sp_share_photos = "Share Photos",
  sp_intro_view_sender_tip = "Share photos with\n your friend during the call.",
  sp_intro_view_receiver_tip = "is selecting photos.",
  sp_scroll_indicator = "Slide to view next photo",
  sp_scroll_page_of = "of",
  sp_share_more_button = "Share More Photos",
  sp_exit_dialog_yes = "Yes",
  sp_exit_dialog_no = "No",
  sp_exit_dialog_desc = "End photo sharing and return to the call?",
  sp_warning_can_not_add_more = "Can't share more than",
  sp_photos = "photos",
  sp_peer = "Your friend",
  sp_test = "test",
}

local supported_languages = {en= true, de = true, es = true, fr = true, it = true, tr = true, ru = true}



local localize_cache = {}

function localize(text)
  if supported_languages[LANGUAGE] then    
    if localize_cache[text] ~= nil then
      return localize_cache[text]
    else
      local ret = MOAILocalization and MOAILocalization.getLocalizedText(text) 
      if (ret == nil or ret == "" or ret == text) and english_string_table[text] ~= nil then 
        ret = english_string_table[text]
      end  
      localize_cache[text] = ret
      print("localize(" .. text .. ") = " .. ret)
      return ret
    end
  else
    return english_string_table[text] or text
  end
end

print ("Language Code : ", LANGUAGE)

localize("sp_intro_view_button")
localize("sp_intro_view_sender_tip")
localize("sp_intro_view_receiver_tip")
localize("sp_scroll_indicator")
localize("sp_scroll_page_of")
localize("sp_share_more_button")
localize("sp_exit_dialog_yes")
localize("sp_exit_dialog_no")
localize("sp_exit_dialog_desc")
localize("sp_warning_can_not_add_more")
localize("sp_photos")
localize("sp_peer")
localize("sp_share_photos")
localize("sp_test")